# Encoder for OpenLaserTag IR transmitter using synchronous code
# Sony SIRC protocol.
# Tomas Krejci [Njord]

from micropython import const
from olt_lib.ir_tx import IR

# Bit reverse a 32 bit value
def rbit32(v):
    v = (v & 0x0000ffff) << 16 | (v & 0xffff0000) >> 16
    v = (v & 0x00ff00ff) << 8 | (v & 0xff00ff00) >> 8
    v = (v & 0x0f0f0f0f) << 4 | (v & 0xf0f0f0f0) >> 4
    v = (v & 0x33333333) << 2 | (v & 0xcccccccc) >> 2
    return (v & 0x55555555) << 1 | (v & 0xaaaaaaaa) >> 1
class LT_ABC(IR):

    def __init__(self, pin, bits, freq, verbose):
        super().__init__(pin, freq, 3 + bits * 2, 40, verbose) # 30 -> 40
        if bits != 24:
            raise ValueError('OLT only support 24 bits.')
        #if bits not in (20, 24):
        #    raise ValueError('bits must be 20 or 24.')
        self.bits = bits
        


    def tx(self, tx1, tx2, tx3):
        bits = self.bits
        v = tx3 & 0xff
        v |= (tx2 & 0xff) << 8
        v |= (tx1 & 0xff) << 16

        v = rbit32(v)
        v = v >> 8

        self.append(2400, 600)
        for _ in range(bits):
            self.append(1200 if v & 1 else 600, 600)
            v >>= 1

# OLT specifies 56KHz
class LT_24(LT_ABC):
    valid = (0xff, 0xff, 0xff)  # Max tx1, tx2, tx3
    def __init__(self, pin, freq=56000, verbose=False):
        super().__init__(pin, 24, freq, verbose)
